#ifndef NEON_H
#define NEON_H
#ifdef __cplusplus
extern "C" 
{
#endif

#ifdef _WIN32
#define DLLEXPORT __declspec(dllexport)
#elif __linux
#define DLLEXPORT
#endif
/*
This contaions the GIE Series Card SDK, Structure Class Function declarations & definitions.
*/

#ifndef NEON_DITRIGGER_CALLBACK_
#define NEON_DITRIGGER_CALLBACK_
typedef void (*NEON_DITRIGGER_CALLBACK)(unsigned int index, int currentState);
#endif

#ifndef NEONDIO_CONFIG_TYPE
#define NEONDIO_CONFIG_TYPE
enum DIO_CONFIGTYPE
{
    DIO_CONFIGTYPE_SNAPSHOT=0,
    DIO_CONFIGTYPE_DIGITAL,
    DIO_CONFIGTYPE_MAXIMUM
};

/*The follwoing configuration is used for old sdk version*/
enum DI_CONFIGTYPE
{
    DI_CONFIGTYPE_TRIGGER=0,
    DI_CONFIGTYPE_DIGITAL,
	DI_CONFIGTYPE_BYPASSDIGITAL,
    DI_CONFIGTYPE_MAXIMUM
};

enum DO_CONFIGTYPE
{
    DO_CONFIGTYPE_STROBEOUT=0,
    DO_CONFIGTYPE_DIGITAL,
	DO_CONFIGTYPE_BYPASSDIGITAL,
    DO_CONFIGTYPE_MAXIMUM
};

#endif

#ifndef DI_TRIGGERPOLARITY_
#define DI_TRIGGERPOLARITY_
enum DI_TRIGGERPOLARITY
{
    DI_TRIGGERPOLARITY_FALLING=0,
    DI_TRIGGERPOLARITY_RISING,
    DI_TRIGGERPOLARITY_BOTH,
	DI_TRIGGERPOLARITY_MAXIMUM
};
#endif

/*
@brief Return value definitions

Return values when a error occurs 
*/
#define FUN_UNSUPPORT -99
#define COMMON_ERROR -1
#define COMMON_BUSY -2

/* 
This function retrieve the device information.
@param sVersion - Output the device information
@param iSize - the size of sVersion
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDeviceInfo(char* sVersion, int iSize);

//DIO
/* 
This function is for setting the DI trigger callback function.
When the DI is triggered, the callback function will be executed.
Before use this function, please confirm your product has supported this.
@param callback - the NEON_DITRIGGER_CALLBACK function pointer
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDITriggerCallback(NEON_DITRIGGER_CALLBACK callback);

/* 
This function retrieve the numbers of DI.
@param iCounts - Output the numbers of DI
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDINums(unsigned int* iCounts);

/* 
This function retrieve the numbers of DO.
@param iCounts - Output the numbers of DO
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDONums(unsigned int* iCounts);

/* 
This function set the DO state.
@param iIndex - The index of DO (zero-based)
@param iState - The state for set
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDOState(unsigned int iIndex,int iState);

/* 
This function retrieve the DO states list.
@param iIndex - The index of DO (zero-based)
@param iState - Output the DO state
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDOState(unsigned int iIndex,int* iState);

/* 
This function retrieve the DI states list.
@param iIndex - The index of DI (zero-based)
@param iState - Output the DI state
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDIState(unsigned int iIndex,int* iState);

/* 
This function set the DIO0 config type.
@param iCfg_type - The config type for set, refer DIO_CONFIGTYPE
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDIO0Config(int iCfg_type);

/* 
This function get the DIO0 config type.
@param iCfg_type - Output the config type, refer DIO_CONFIGTYPE
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDIO0Config(int* iCfg_type);

/* 
This function set the DI trigger polarity.
@param iIndex - The index of DI (zero-based)
@param iPositiveTriggerPolarity - The trigger polarity for setting, refer DI_TRIGGERPOLARITY
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDITriggerPolarity(unsigned int iIndex, int iPositiveTriggerPolarity);

/* 
This function get the DI trigger polarity.
@param iIndex - The index of DI (zero-based)
@param iPositiveTriggerPolarity - Output the trigger polarity, refer DI_TRIGGERPOLARITY
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDITriggerPolarity(unsigned int iIndex, int* iPositiveTriggerPolarity);

//Camera Control
/*
This function is enable/disable Strobe out.
@param iEnable - 0 is disable or others enable
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetStrobeOutEnable(int iEnable);

/*
This function is get the enable state of the Strobe out.
@param iEnable - return the enable state
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetStrobeOutEnable(int* iEnable);

/* 
This function set Strobe Out delay.(microseconds)
@param iMicroSecond - Microseconds, set 0 is 1 microsecond
@return - Return true if success or COMMON_ERROR if failed.
*/              
DLLEXPORT int Neon_SetStrobeOutDelay(unsigned int iMicroSecond);

/*
This function get Strobe Out delay.(microseconds)
@param iMicroSecond - output Microseconds, set 0 is 1 microsecond
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetStrobeOutDelay(unsigned int* iMicroSecond);

/* 
This function set Strobe Out Pulse width.(microseconds)
@param iMicroSecond - Microseconds, set 0 is 1 microsecond
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetStrobeOutPulseWidth(unsigned int iMicroSecond);

/*
This function get Strobe Out Pulse width.(microseconds)
@param iMicroSecond - output Microseconds, set 0 is 1 microsecond
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetStrobeOutPulseWidth(unsigned int* iMicroSecond);

/* 
This function set the inverted polarity of Strobe Out. If set 1, the output polarity will be inverted.( default is 0)
@param iInvertedPolarity - Invert the polarity of strobe out. (0 is non-inverted polarity, 1 is inverted polarity)
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetStrobeOutInvertedPolarity(int iInvertedPolarity);

/*
This function get the inverted polarity value of Strobe Out.( default is 0)
@param iInvertedPolarity - Invert the polarity of strobe out. (0 is non-inverted polarity, 1 is inverted polarity)
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetStrobeOutInvertedPolarity(int* iInvertedPolarity);


/* 
This function set Exposure out delay.(microseconds)
@param iMicroSecond - Microseconds, 0 is no delay
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetExposureDelay(unsigned int iMicroSecond);

/*
This function get Exposure out delay.(microseconds)
@param iMicroSecond - output Microseconds, 0 is no delay
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetExposureDelay(unsigned int* iMicroSecond);

/*
This function set Debounce filter time. (unit is clock cycle time: 40 ns)
@param iNtimes40ns - the nimbers of clock cycle
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDebounceTime(unsigned int iNtimes40ns);

/*
This function get Debounce filter time. (unit is clock cycle time: 40 ns)
@param iNtimes40ns - output the nimbers of clock cycle
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDebounceTime(unsigned int* iNtimes40ns);


//*********************The following functions are not used anymore*********************
/* 
This function set the DO0 config type.
@param iCfg_type - The config type for set
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDO0Config(int iCfg_type);

/* 
This function get the DO0 config type.
@param iCfg_type - Output the config type
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDO0Config(int* iCfg_type);

/* 
This function set the DI0 config type.
@param iCfg_type - The config type for set
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_SetDI0Config(int iCfg_type);

/* 
This function get the DI0 config type.
@param iCfg_type - Output the config type
@return - Return true if success or COMMON_ERROR if failed.
*/
DLLEXPORT int Neon_GetDI0Config(int* iCfg_type);

#ifdef __cplusplus
}
#endif
#endif // NEON_H
